package ca.mcgill.sable.abc.impact.visualizer.actions;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.ajdt.core.BuildConfig;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.core.search.IJavaSearchScope;
import org.eclipse.jdt.core.search.SearchEngine;
import org.eclipse.jdt.internal.ui.util.MainMethodSearchEngine;
import org.eclipse.jdt.ui.JavaUI;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.operation.IRunnableContext;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IActionDelegate;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.console.MessageConsoleStream;
import org.eclipse.ui.dialogs.ResourceSelectionDialog;
import org.eclipse.ui.dialogs.SelectionDialog;

import ca.mcgill.sable.abc.impact.visualizer.console.ImpactConsoleFactory;
import ca.mcgill.sable.abc.impact.visualizer.model.ImpactFactory;
import ca.mcgill.sable.abc.impact.visualizer.util.ImpactConstants;
import ca.mcgill.sable.abc.impact.visualizer.views.ImpactView;

public class RunImpactAnalysisActionDelegate implements IObjectActionDelegate {

	private ISelection selection;
	private static IProject currentProject;
	private IWorkbenchPart targetPart;
	
	public RunImpactAnalysisActionDelegate() {
		super();
	}

	/**
	 * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
	 */
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
		this.targetPart = targetPart;
	}

	/**
	 * @see IActionDelegate#run(IAction)
	 */
	public void run(IAction action) {
		
		if (! (this.selection instanceof IStructuredSelection)) return;
		IStructuredSelection selection = (IStructuredSelection)this.selection;
		final IProject project = (IProject)selection.getFirstElement();
		final IJavaProject jProject = JavaCore.create(project);
		
		//TODO so far, AJDT only compiles files returned by BuildConfig.getIncludedSourceFiles
		//but ajc supports -sourceroots, if in the future AJDT supports this, add here
		List<IFile> sourceFiles = BuildConfig.getIncludedSourceFiles(project);

		IRunnableContext context = targetPart.getSite().getWorkbenchWindow();
		final Shell shell = targetPart.getSite().getShell();
		IJavaSearchScope searchScope = SearchEngine.createJavaSearchScope(
				new IJavaElement[]{jProject}, IJavaSearchScope.SOURCES);
		int flag = 0;
		int numMain = 2;
		String mainClassName = null;
		
		try {
			MainMethodSearchEngine engine= new MainMethodSearchEngine();
			IType[] retTypes = engine.searchMainMethods(context, searchScope, flag);
			numMain = retTypes.length;
			if (numMain == 1) {
				mainClassName = ((IType)retTypes[0]).getFullyQualifiedName();
			}
		} catch (InvocationTargetException e1) {
			e1.printStackTrace();
		} catch (InterruptedException e1) {
			e1.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		if (numMain > 1) {
			SelectionDialog dialog = JavaUI.createMainTypeDialog(
					shell, context, searchScope, flag, false);
			dialog.setTitle("Select Main Class");
			dialog.setMessage("Please select the main class:");
			int retCode = dialog.open();
			if (retCode == ResourceSelectionDialog.OK) {
				Object[] result = dialog.getResult();
				mainClassName = ((IType)result[0]).getFullyQualifiedName();
			} else {
				return;
			}
		}
		
		IFolder outputFolder = project.getFolder(ImpactConstants.OUTPUT_FOLDER);
		try {
			outputFolder.delete(true, null);
			outputFolder.create(true, true, null);
		} catch (CoreException e) {
			e.printStackTrace();
			return;
		}
		
		/* Unfortunately, AJDT greedily includes all .java and .aj files, even inside
		 * a hidden folder, so abondand this idea
		 * the reason to consider this is that impact analysis is long-time run,
		 * during it running resources may have changed already, copy keep a hot-copy
		 * in tmp folder and process tmp folder, but...		
		IFolder tmpFolder = project.getFolder(AbcImpactConstants.TMP_FOLDER_NAME);

		// may improve here to only add/remove changed source files 
		// refreshing the tmp folder
		try {
			tmpFolder.delete(true, null);
			tmpFolder.create(true, true, null);
			for (IFile sourceFile : sourceFiles) {
				if (sourceFile == null) continue;
				IPath sourcePath = sourceFile.getProjectRelativePath();
				IFolder tmpSourceFolder = project.getFolder(
						(AbcImpactConstants.TMP_FOLDER + File.separator +
						sourcePath.removeLastSegments(1)));
				Stack<IFolder> s = new Stack<IFolder>();
				IFolder temp = tmpSourceFolder;
				while (! temp.exists()) {
					s.push(temp);
					temp = (IFolder)temp.getParent();
				}
				while(! s.empty()) {
					IFolder f = s.pop();
					f.create(true, true, null);
				}
				
				IPath tmpSourcePath = tmpFolder.getFullPath().append(sourcePath);
				sourceFile.copy(tmpSourcePath, true, null);
			}
		} catch (CoreException e) {
			e.printStackTrace();
			return;
		} */
		
		ArrayList<String> argsList = new ArrayList<String>(10+sourceFiles.size());
		
		if (mainClassName != null) {
			argsList.add(ImpactConstants.ABC_OPTION_MAIN_CLASS);
			argsList.add(mainClassName);
		}
		argsList.add(ImpactConstants.ABC_OPTION_EXT);
		argsList.add(ImpactConstants.ABC_OPTION_IMPACT_NAME);
		argsList.add(ImpactConstants.ABC_OPTION_D);
		argsList.add(outputFolder.getLocation().toOSString());
		StringBuffer cpStringb = new StringBuffer();
		try {
			IClasspathEntry[] cpEntries = jProject.getResolvedClasspath(true);
			for (IClasspathEntry cpEntry : cpEntries) {
				//TODO deal with other kinds of entry
				if (cpEntry.getEntryKind() == IClasspathEntry.CPE_LIBRARY ) {
					cpStringb.append(cpEntry.getPath().toOSString());
					cpStringb.append(File.pathSeparatorChar);
				}
			}
		} catch (JavaModelException e1) {
			e1.printStackTrace();
		}
		if (cpStringb.length() > 0) {
			argsList.add(ImpactConstants.ABC_OPTION_CP);
			argsList.add(cpStringb.toString());
		}
		//TODO deal with aspectj inpath and aspectpath
		// see ..ajdt.core.AspectJCorePreference.getxx

		
		// source files
		for (IFile sourceFile : sourceFiles) {
				if (sourceFile == null) continue;
				argsList.add(sourceFile.getLocation().toOSString());
		}

		final String[] args = argsList.toArray(new String[argsList.size()]);
		final IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
		final ImpactView stateImpactView = (ImpactView)page.findView(ImpactView.ID);
		
		// -- run in background start
		Job impactJob = new Job("Impact Analysis") {
			@Override
			protected IStatus run(final IProgressMonitor monitor) {
				try {
					
					monitor.beginTask(getName()+"...", 100);

					// ImpactConsoleFactory.showConsole(); //TODO useless?
					
					abc.main.Main.reset(); // TODO remove comment
					monitor.worked(15);
					
					abc.main.Main.main(args); // TODO remove comment
					monitor.worked(80);

					final IProject preProject = currentProject;
					currentProject = project;

					// let user see the progress bar
					for (int i = 0; i < 3; i++) {
						Thread.sleep(1000);
						monitor.worked(1);
					}
					
					// the followings should be run in the UI thread
					Thread reloadThread = new Thread() {
						public void run() {
							
							// delete markers for previous project
							if (preProject != null) {
								ImpactFactory.deleteAllMarkers(preProject);
							}
							// delete markers for current project
							if (currentProject != null) {
								ImpactFactory.deleteAllMarkers(currentProject);
							}
							
							ImpactFactory.populateTreeAndMarker();
							monitor.worked(1);
							if (stateImpactView != null) {
								stateImpactView.reload();
							}
							try {
								page.showView(ImpactView.ID);
							} catch (PartInitException e) {
								e.printStackTrace();
							}
						}
					};
					shell.getDisplay().syncExec(reloadThread);
					monitor.done();
					
					return Status.OK_STATUS;
				} catch (Exception e) {
					e.printStackTrace();
				}
				return Status.CANCEL_STATUS;
			}
		};
		impactJob.setPriority(Job.SHORT);
		impactJob.setUser(true);
		impactJob.schedule();
		// -- run in background end
		
		
//		// -- run in foreground
//		try {
//			abc.main.Main.reset(); 
//			abc.main.Main.main(args);
//
//			if (currentProject != null) {
//				ImpactFactory.deleteAllMarkers(currentProject);
//			}
//			currentProject = project;
//
//			ImpactFactory.populateData();
//			if (stateImpactView != null) {
//				stateImpactView.reload();
//			} else {
//				page.showView(ImpactView.ID);
//			}
//		} catch (Exception e) {
//			e.printStackTrace();
//		}
//		// -- run in foreground end
	}

	/**
	 * @see IActionDelegate#selectionChanged(IAction, ISelection)
	 */
	public void selectionChanged(IAction action, ISelection selection) {
		this.selection = selection;
	}
	
	public static IProject getCurrentProject() {
		return currentProject;
	}

}
