package ca.mcgill.sable.abc.impact.visualizer.model;

import java.util.List;

import org.eclipse.core.runtime.Platform;
import org.eclipse.jdt.core.IJavaElement;

import ca.mcgill.sable.abc.impact.visualizer.actions.RunImpactAnalysisActionDelegate;
import ca.mcgill.sable.abc.impact.visualizer.util.ImpactPluginUtil;

import soot.SootMethod;
import soot.Type;

import abc.impact.impact.LookupImpact;
import abc.impact.impact.LookupMethod;
import abc.impact.impact.MethodSignature;

public final class LookupImpactAffectedTypeTreeNode extends AbstractTreeNode
		implements IActJavaElement {

	final private LookupImpact lookupImpact;
	private final IJavaElement jEle;
	
	public LookupImpactAffectedTypeTreeNode(ITreeNode parent, final LookupImpact lookupImpact) {
		super(parent);
		this.lookupImpact = lookupImpact;
		
		LookupMethod lookupMethod = lookupImpact.getLookupMethod();
		StringBuffer nameb = new StringBuffer();
		nameb.append('[');
		nameb.append(lookupMethod.getType().getName());
		nameb.append("]");
		MethodSignature mSig = lookupMethod.getMSig();
		if (! mSig.getMethodName().equals(SootMethod.constructorName)) {
			nameb.append(".");
			nameb.append(mSig.getMethodName());
		}
		nameb.append('(');
		List<Type> params = mSig.getParams();
		for (Type type : params) {
			nameb.append(type);
			nameb.append(',');
		}
		if (!params.isEmpty()) nameb.setLength(nameb.length()-1);
		nameb.append(')');
		this.name = nameb.toString();
		
		this.image = ImageFactory.LOOKUP_METHOD.getImage();
		
		this.jEle = ImpactPluginUtil.findJavaType(
				RunImpactAnalysisActionDelegate.getCurrentProject(),
				lookupImpact.getLookupMethod().getType().getName());
	}

	public LookupImpact getLookupImpact() {
		return lookupImpact;
	}

	public IJavaElement getJavaElement() {
		return jEle;
	}

	public Object getAdapter(Class adapter) {
		return Platform.getAdapterManager().getAdapter(this, adapter);
	}

	public boolean provideJavaElementImage() {
		return false;
	}

	public boolean provideJavaElementText() {
		return false;
	}
}
