package ca.mcgill.sable.abc.impact.visualizer.views;

import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.ui.JavaElementLabelProvider;
import org.eclipse.jface.viewers.DecoratingLabelProvider;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.LabelProviderChangedEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.PlatformUI;

import ca.mcgill.sable.abc.impact.visualizer.model.IProvideJavaElementImage;
import ca.mcgill.sable.abc.impact.visualizer.model.IProvideJavaElementText;
import ca.mcgill.sable.abc.impact.visualizer.model.ITreeNode;

public final class ImpactViewLabelProvider extends LabelProvider {

	private ILabelProvider jLabelProvider;
	private boolean addedListener = false;
	
	public ImpactViewLabelProvider() {
		jLabelProvider = new DecoratingLabelProvider(
				new JavaElementLabelProvider(JavaElementLabelProvider.SHOW_DEFAULT |
						JavaElementLabelProvider.SHOW_SMALL_ICONS), PlatformUI
						.getWorkbench().getDecoratorManager().getLabelDecorator());
	}
	
	@Override
	public void addListener(ILabelProviderListener listener) {
		super.addListener(listener);
		if (!addedListener) {
			addedListener = true;
			// as we are only retrieving images from labelProvider not using it
			// directly, we need to update this label provider whenever that one
			// updates
			jLabelProvider.addListener(new ILabelProviderListener() {
				public void labelProviderChanged(LabelProviderChangedEvent event) {
					fireLabelChanged();
				}
			});
		}
	}

	private void fireLabelChanged() {
		Object[] listeners = getListeners();
		if (listeners.length > 0 ) {
			LabelProviderChangedEvent event = new LabelProviderChangedEvent(this);
			for (Object listener : listeners) {
				((ILabelProviderListener)listener).labelProviderChanged(event);
			}
		}
	}
	
	@Override
    public String getText(Object element) {
		ITreeNode node = (ITreeNode) element;
		
		if (node instanceof IProvideJavaElementText) {
	    	IJavaElement jElement = ((IProvideJavaElementText)node).getJavaElement();
	    	if (jElement != null) {
	    		return jLabelProvider.getText(jElement);
	    	}
		}
		return node.getName();
	}

	@Override
	public Image getImage(Object element) {
		ITreeNode node = (ITreeNode) element;
		Image image = node.getImage();
		
		if (node instanceof IProvideJavaElementImage) {
	    	IJavaElement jElement = ((IProvideJavaElementImage)node).getJavaElement();
	    	if (jElement != null) {
	    		image = jLabelProvider.getImage(jElement);
	    	}
		}
		
		Image decorated = PlatformUI.getWorkbench().getDecoratorManager()
				.decorateImage(image, element);
		if (decorated != null) return decorated;
		return image;
	}

	@Override
	public void removeListener(ILabelProviderListener listener) {
		super.removeListener(listener);
	}
}
