package ca.mcgill.sable.abc.impact.visualizer.views;

import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;

import abc.impact.impact.ReferredPlace;
import abc.impact.utils.PositionComparator;
import ca.mcgill.sable.abc.impact.visualizer.model.AdviceDeclTreeNode;
import ca.mcgill.sable.abc.impact.visualizer.model.ComputationStringTreeNode;
import ca.mcgill.sable.abc.impact.visualizer.model.DirectStateStringTreeNode;
import ca.mcgill.sable.abc.impact.visualizer.model.IndirectStateStringTreeNode;
import ca.mcgill.sable.abc.impact.visualizer.model.LookupImpactCurrentTypeTreeNode;
import ca.mcgill.sable.abc.impact.visualizer.model.LookupImpactOriginTypeTreeNode;
import ca.mcgill.sable.abc.impact.visualizer.model.LookupStringTreeNode;
import ca.mcgill.sable.abc.impact.visualizer.model.ReferredPlaceTreeNode;
import ca.mcgill.sable.abc.impact.visualizer.model.ShadowingImpactCurrentTypeTreeNode;
import ca.mcgill.sable.abc.impact.visualizer.model.ShadowingImpactOriginTypeTreeNode;
import ca.mcgill.sable.abc.impact.visualizer.model.ShadowingStringTreeNode;
import ca.mcgill.sable.abc.impact.visualizer.model.StateStringTreeNode;

public class ImpactViewSorter extends ViewerSorter {

	@Override
	public int category(Object element) {
		
		if (element instanceof StateStringTreeNode) return 4;
		if (element instanceof ComputationStringTreeNode) return 3;
		if (element instanceof ShadowingStringTreeNode) return 1;
		if (element instanceof LookupStringTreeNode) return 2;
		
		if (element instanceof DirectStateStringTreeNode) return 1;
		if (element instanceof IndirectStateStringTreeNode) return 2;
		
		if (element instanceof ReferredPlaceTreeNode) {
			ReferredPlaceTreeNode rpNode = (ReferredPlaceTreeNode) element;
			if (rpNode.getReferredPlace() == ReferredPlace.CLASS) return 1;
			if (rpNode.getReferredPlace() == ReferredPlace.PACKAGE) return 2;
			if (rpNode.getReferredPlace() == ReferredPlace.PROTECTED) return 3;
			if (rpNode.getReferredPlace() == ReferredPlace.OTHER) return 4;
		}
		
		if (element instanceof ShadowingImpactOriginTypeTreeNode) return 1;
		if (element instanceof ShadowingImpactCurrentTypeTreeNode) return 2;
		
		if (element instanceof LookupImpactOriginTypeTreeNode) return 1;
		if (element instanceof LookupImpactCurrentTypeTreeNode) return 2;
		
		return super.category(element);
	}

	@Override
	public int compare(Viewer viewer, Object e1, Object e2) {
		
		if (e1 instanceof AdviceDeclTreeNode && e2 instanceof AdviceDeclTreeNode) {
			AdviceDeclTreeNode n1 = (AdviceDeclTreeNode)e1;
			AdviceDeclTreeNode n2 = (AdviceDeclTreeNode)e2;
			
			return PositionComparator.getInstance().compare(
					n1.getAdviceDecl().getPosition(), 
					n2.getAdviceDecl().getPosition());
		}
		
		return super.compare(viewer, e1, e2);
	}

}
